/*
 * I2C.c
 *
 *  Created on: 25.01.2015
 *      Author: DL9MWE
 *      06.02.2015
 */
#include "main.h"

uint16_t ui16I2C_Start_write8Bit_Stop(unsigned char I2C_device_address, unsigned char value)
{
	uint16_t error_code;
	vI2Cll_set_start_condition();				// Send I2C Start Condition
	vI2Cll_txByte(I2C_device_address);			// [ADDR] + R/W bit = 0
	error_code = ui16I2Cll_GetSlave_ack();		// get slave acknowledge (ACKS)
	vI2Cll_txByte(value);
	error_code += ui16I2Cll_GetSlave_ack();		// get ACKS
	vI2Cll_set_stop_condition();
	return error_code;
}

uint8_t ui8I2C_Start_read8Bit_Stop(unsigned char I2C_device_address, uint16_t *error_code)
{
	uint8_t rxData;
	vI2Cll_set_start_condition();				// Send I2C Start Condition
	vI2Cll_txByte(I2C_device_address+1);		// [ADDR] + R/W bit = 1
	*error_code = ui16I2Cll_GetSlave_ack();		// get slave acknowledge (ACKS)
	rxData = ui8I2Cll_rxByte();
	vI2Cll_SendMaster_nack();					// Master negative Acknowledge
	vI2Cll_set_stop_condition();
	return rxData;
}


// adressiert das I2C-Device und setzt eine Register-adresse
void vI2C_set_register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
	vI2Cll_set_start_condition();				// Send I2C Start Condition
	vI2Cll_txByte(I2C_device_address);		// [ADDR] + R/W bit = 0
	*error_code = ui16I2Cll_GetSlave_ack();		// get slave acknowledge (ACKS)
	vI2Cll_txByte(I2C_register_address);	// send ADXL345B Register Address
	*error_code += ui16I2Cll_GetSlave_ack();		// get ACKS
}

// Byte aus I2C-Device-Register lesen
uint8_t ui8I2C_read_char(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
  uint8_t data;
	vI2C_set_register(I2C_device_address, I2C_register_address, error_code);
	vI2Cll_set_stop_condition();
	vI2Cll_set_start_condition();				// repeated start condition
	vI2Cll_txByte(I2C_device_address + 1);	// [ADDR] + R/W bit = 1
	*error_code = ui16I2Cll_GetSlave_ack();		// get ACKS
	data = ui8I2Cll_rxByte();
	return data;
}

// liest ein Byte aus einem Register des I2C-Devices aus
uint8_t ui8I2C_read8Bit_Register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int *error_code) {
  unsigned char data;
	data = ui8I2C_read_char(I2C_device_address, I2C_register_address, error_code);
	vI2Cll_SendMaster_nack();
	vI2Cll_set_stop_condition();
	return data;
}


// schreibt ein Byte in ein Register des I2C-Devices
uint16_t ui16I2C_write_register_value(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned char register_value) {
	unsigned int error_code;
	vI2C_set_register(I2C_device_address, I2C_register_address, &error_code);
	vI2Cll_txByte(register_value); 					// write register value
	error_code += ui16I2Cll_GetSlave_ack();			// get ACKS
	vI2Cll_set_stop_condition();
	return error_code;
}
// liest ein Byte aus einem Register des I2C-Devices aus
uint8_t ui8I2C_read_8Bit_Register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int *error_code) {
  unsigned char data;
	data = ui8I2C_read_char(I2C_device_address, I2C_register_address, error_code);
	vI2Cll_SendMaster_nack();
	vI2Cll_set_stop_condition();
	return data;
}
// liest mehrere (min. 2) Bytes aus aufeinanderfolgenden Registern des I2C-Devices aus
uint16_t ui16I2C_read_multiple_chars(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int numBytes, unsigned char* rxData) {
	unsigned int error_code;
	unsigned int index=1;

	rxData[0] = ui8I2C_read_char(I2C_device_address, I2C_register_address, &error_code);
	vI2Cll_SendMaster_ack();
	while(1) {
		rxData[index] = ui8I2Cll_rxByte();
		if(++index < numBytes)	vI2Cll_SendMaster_ack();
		else {
			vI2Cll_SendMaster_nack();
			break;
		}
	}
	vI2Cll_set_stop_condition();
	return error_code;
}


// Ende I2C.c
