/*
 * LCD_I2C.c
 *
 *  Created on: 25.07.2012
 *      Author: Gerrit
  *  adaptiert fr I2C-Ansteuerung via PCF8574A 21.10.2014 DL9MWE
 *  09.02.2015
 */

#include "main.h"


//unsigned char ucLCD_x, ucLCD_y, ucLCD_maxx;
unsigned int const uiBase_y[4] = { 0x80, 0xc0, 0x94, 0xd4 }; // example for 4x20 display
// 1. line 0x00..0x13 + Bit7==0x80
// 2. line 0x40..0x53 + Bit7==0xc0
// 3. line 0x14..0x27 + Bit7==0x94
// 4. line 0x54..0x67 + Bit7==0xd4
// BIT7 nessesary for command "Set DD RAM address"


//********************************************************************************
//Description  : Sends byte to lcd in 4bit-mode
//Input        : data - data to send
//             : reg - specifies the lcd-register
//Output       : none
//********************************************************************************
void vSendByteToLCD_4BIT(unsigned char ucData, unsigned char ucReg) {
// Data valid at HIGH-LOW transition of enable signals !!
// High nibble (BIT7-BIT4) are transfered first
	unsigned char LCD_byte;

	LCD_byte = ucData >> 4;		// High nibble first
	if(ucReg == COMMAND)	LCD_byte &= ~RS_BIT;
	else					LCD_byte |= RS_BIT;
	LCD_byte |= ENABLE_BIT;
	ui16PCF8574A_out(LCD_byte);
	_delay_us(50);		//vLCD_delay(1);
	LCD_byte &= ~ENABLE_BIT;
	ui16PCF8574A_out(LCD_byte);
	_delay_us(50);		//vLCD_delay(1);

	LCD_byte = ucData & 0x0f;	// Low nibble
	if(ucReg == COMMAND)	LCD_byte &= ~RS_BIT;
	else					LCD_byte |= RS_BIT;
	LCD_byte |= ENABLE_BIT;
	ui16PCF8574A_out(LCD_byte);
	_delay_us(50);		//vLCD_delay(1);
	LCD_byte &= ~ENABLE_BIT;
	ui16PCF8574A_out(LCD_byte);
	_delay_us(50);		//vLCD_delay(1);
}

// ********************************************************************************
// Description  : Sends byte to the LCD's instruction-register in 8bit-mode
// Input        : data - data to send
// Output       : none
// ********************************************************************************
void vSendByteToLCD_8BIT(unsigned char ucData) {
// Data valid at HIGH-LOW transition of enable signals !!
// for this function only control-registers are written
	unsigned char LCD_byte;

	LCD_byte = ucData >> 4;
	LCD_byte &= ~RS_BIT;
	LCD_byte |= ENABLE_BIT;
	ui16PCF8574A_out(LCD_byte);
	_delay_us(50);		//vLCD_delay(1);
	LCD_byte &= ~ENABLE_BIT;
	ui16PCF8574A_out(LCD_byte);
	_delay_us(50);		//vLCD_delay(1);
}

//********************************************************************************
//Description  : initialize the LCD controller
//             : needs about 15ms to start
//Input        : lcd_columns	- count of columns
//Output       : Ja: LCD via I2C OK, sonst = Nein
//********************************************************************************
Ja_Nein_t jnLCD_init() {
//wait for more than 40ms after Vcc rises to 2.7V
//wait for more than 15ms after Vcc rises to 4.5V
	volatile unsigned int i;
	_delay_ms(42);
	ui16PCF8574A_init();
	if(ui16PCF8574A_out(0xC0) != 0) // select command-register, ENABLE low, Bit7 = Bit6 = 1 (PCF8574A-Input)
		return Nein;

	_delay_ms(15);	//vLCD_delay(300);	//300x50us=15ms
	vSendByteToLCD_8BIT(0x30); //function set (Interface 8bit)
	_delay_ms(5);	//vLCD_delay(82);	//82x50us=4.1ms
	vSendByteToLCD_8BIT(0x30); //function set (Interface 8bit)
	_delay_us(150);	//vLCD_delay(3);		//3x50us=150us
	vSendByteToLCD_8BIT(0x30); //function set (Interface 8bit)
	_delay_us(50);		//vLCD_delay(1);
	vSendByteToLCD_8BIT(0x20); //function set 4bit (Interface 8bit)
	_delay_us(50);		//vLCD_delay(1);
	vSendByteToLCD_4BIT(0x28, COMMAND); //function set + number of display lines + character font
	_delay_us(50);		//vLCD_delay(1);

	vLCD_clear(); //clear display
	vSendByteToLCD_4BIT(0x06, COMMAND); //entry mode set, Cursor moves, no Display shift
	return Ja;
}

//********************************************************************************
// Description  : set the LCD display position x=1..16 y=1..2
//              :
// Input        : x - X-position
//              : y - Y-position
// Output       : none
//********************************************************************************
void vLCD_gotoxy(unsigned char ucX, unsigned char ucY) {
	vSendByteToLCD_4BIT(uiBase_y[(unsigned int)ucY-1] + ucX-1, COMMAND);
}

//********************************************************************************
// Description  : clears the LCD
// Input        : none
// Output       : none
//********************************************************************************
void vLCD_clear(void) {
	volatile unsigned int i;

	vSendByteToLCD_4BIT(0x0c, COMMAND); //Display ON, Cursor and Blinking off
	_delay_us(50);
	vSendByteToLCD_4BIT(0x01, COMMAND); //function display clear
	_delay_ms(2);						//wait for clear operation (>=1.52ms)
	vLCD_gotoxy(1,1);

}

//********************************************************************************
// Description  : write a char to the LCD
// Input        : c - character
// Output       : none
//********************************************************************************
void vLCD_putc(unsigned char ucC) {
	_delay_us(50);
	vSendByteToLCD_4BIT(ucC, DATA);
}

//********************************************************************************
// Description  : write the string str to the LCD
// Input        : *str	pointer to string
// Output       : none
//********************************************************************************
void vLCD_puts(unsigned char *ucStr) {
	while (*ucStr != 0x0)
		vLCD_putc(*ucStr++);
}

void vLCD_putnC(unsigned char *ucStr, unsigned int count) {
	while (count--)
		vLCD_putc(*ucStr++);
}

// Hilfsfunktion: gibt eine Stelle auf dem LCD aus (fhrende Nullen werden ignoriert)
unsigned char put_uidigit(unsigned int *uiX, unsigned char stelle,
		unsigned char print) {
	unsigned int y, dezimal = 1;
	while (stelle) {
		dezimal = dezimal * 10;
		--stelle;
	}
	y = *uiX / dezimal;
	if (y || print) {
		vLCD_putc(y + 0x30);
		print = 1;
		*uiX -= y * dezimal;
	}
	return print;
}

//********************************************************************************
// Description  : write an unsigned integer to LCD
// Input        : unsigned integer uiX
// Output       : none
//********************************************************************************
// vorzeichenlose 16-Bit-Zahl als Dezimal-Wert auf dem Display ausgeben
void vLCD_putui(unsigned int uiX) {
	unsigned char stelle, print = 0;

	for (stelle = 4; stelle; --stelle) {
		print = put_uidigit(&uiX, stelle, print); // fhrende Nullen nicht ausgeben
	}
	vLCD_putc(uiX + 0x30); // die Einerstelle immer ausgeben
}


//********************************************************************************
// Description  : write a signed integer to LCD
// Input        : signed integer iX
// Output       : none
//********************************************************************************
void vLCD_puti(int iX) {
	if (iX < 0) {
		vLCD_putc('-');
		iX = ~iX + 1;
	}
	vLCD_putui((unsigned int) iX);
}

//********************************************************************************
// Description  : write a signed long integer to LCD
// Input        : signed long integer value, Anzahl Stellen gesamt, Anzahl Nachkommastellen
// Output       : none
//********************************************************************************
void vLCD_putli(long value, unsigned int stellen, unsigned int nachkommastellen) {
	unsigned char cbuffer[15];
	cbuffer[(unsigned int)ltoChars(value, cbuffer, nachkommastellen, stellen, Nein/*notrim*/)] = '\x00';
	vLCD_puts(cbuffer);
}

// Ende LCD_I2C.c
