/*
 * SW_I2C_lowlevel.c
 * Software I2C-Schnittstelle - low level Funktionen
 *  Created on: 25.01.2015
 *      Author: DL9MWE
 *      02.02.2015
 */
#include "main.h"

void vI2C_init(void)
{
  PxSEL &= ~(SCL | SDA);                    // Set GPIO function
  PxDIR |= (SCL | SDA);                     // Set output direction
  PxOUT &= ~(SCL | SDA);                    // SCL & SDA = 0 => low when = outputs
  PxDIR &= ~(SCL | SDA);                    // set to input, SCL & SDA = 1
}

// Set up start condition for I2C
void vI2Cll_set_start_condition(void) {
	PxDIR |= SDA;		// Set to output, data low [SDA = 0]
	// Start condition hold time
#ifdef I2C_fastmode
	_NOP();				// Quick delay > 0,6us
#else
	_delay_us(4);
#endif
	PxDIR |= SCL;		// Set to output, clock low [SCL = 0]
	_NOP();
}

// Send I2C stop command
void vI2Cll_set_stop_condition(void) {
	PxDIR |= SDA;                 // Set to output, data low [SDA = 0]
	_NOP();
	_NOP();                       // Quick delay
#if (F_MHZ == 16)
    _NOP();
    _NOP();
#endif
    PxDIR &= ~SCL;                // Set to input, SCL = 1 via pull-up
#ifdef I2C_fastmode
    _NOP();
    _NOP();                       // Quick delay
#else
    _delay_us(4);
#endif
    PxDIR &= ~SDA;                // Set to input, SDA = 1 via pull-up
}

// Get Slave Acknowledge
unsigned int ui16I2Cll_GetSlave_ack(void)
{
  unsigned int error;
   PxDIR &= ~SCL;			// Set to input, SCL = 1 via pull-up
   _delay_us(4);			// delay 4us
   if (PxIN & SDA)
	   error = 1;
   else
	   error = 0;
   PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
   return error;
}

// set I2C_clock_pulse
void vMSP430_SWI2C_clockpulse(void) {
	PxDIR &= ~SCL;			// Set to input, SCL = 1 via pull-up
	// clock high time
#ifdef I2C_fastmode
	_NOP();					// ca. 0,6us delay to meet I2C spec
#else
	_delay_us(4);			// ca. 4us delay to meet I2C spec
#endif
	PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
	_NOP();
}

// Send Master Acknowledge
void vI2Cll_SendMaster_ack(void)
{
	PxDIR |= SDA;			// Set to output, data low [SDA = 0]
	vMSP430_SWI2C_clockpulse();
	PxDIR &= ~SDA;			// Set SDA to input
}

// Send negative acknowledge
void vI2Cll_SendMaster_nack(void)
{
	PxDIR &= ~SDA;			// Set to input, SDA = 1 via pull-up
	vMSP430_SWI2C_clockpulse();
}

// Master sendet ein Byte an den I2C-Slave
void vI2Cll_txByte(unsigned char data) {
  unsigned char bits, temp;

	temp = data;
	bits = 0x08;				// Load I2C bit counter
	while (bits != 0x00) {		// Loop until all bits are shifted
		if (temp & BIT7)		// Test data bit
			PxDIR &= ~SDA;		// Set to input, SDA = 1 via pull-up
		else
			PxDIR |= SDA;		// Set to output, data low [SDA = 0]
		// SDA setup time > 100ns bzw. 250ns
		_NOP();
#if (F_MHZ == 16)
		_NOP();
		_NOP();
#endif
		PxDIR &= ~SCL;			// Set to input, clock high [SCL = 1]
		temp = (temp << 1);		// Shift bits 1 place to the left
		// Clock high time
#ifdef I2C_fastmode
//		_NOP();
		_NOP();					// Quick delay > 0,6us
#else
		_delay_us(4);
#endif
		PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
		bits = (bits - 1);		// Loop until 8 bits are sent
	}
	PxDIR &= ~SDA;				// Set to input, Slave setzt SDA
}

// liest ein Byte via I2C ein
unsigned char ui8I2Cll_rxByte(void) { // Read 8 bits of I2C data
	unsigned char bits, temp = 0;	// I2C bit counter
	bits = 0x08;					// Load I2C bit counter
	while (bits > 0)				// Loop until all bits are read
	{
		PxDIR &= ~SCL;            // Set to input, SCL = 1 via pull-up
#ifdef I2C_fastmode
		_NOP();
#else
		_delay_us(4);
#endif
		temp = (temp << 1);       // Shift bits 1 place to the left
		if (PxIN & SDA)           // Check digital input
			temp = (temp + 1);      // If input is 'H' store a '1'
		PxDIR |= SCL;             // Set to output, data low [SCL = 0]
		bits = (bits - 1);        // Decrement I2C bit counter
		_NOP();
#if (F_MHZ == 16)
		_NOP();
		_NOP();
#endif
	}
	return (temp);               // Return 8-bit data byte
}


// Ende SW_I2C_lowlevel.c
