/*
 * USCIA0_UART.c
 *  UART 9600 Baud, N81
 *  28.09.2014	DL9MWE
 *  07.02.2015
 */
//DL9MWE used: ***************************************************************
//   MSP430G2xx3 Demo - USCI_A0, 9600 UART Echo ISR, DCO SMCLK
//
//   Description: Echo a received character, RX ISR used. Normal mode is LPM0.
//   USCI_A0 RX interrupt triggers TX Echo.
//   Baud rate divider with 1MHz = 1MHz/9600 = ~104.2
//   ACLK = n/a, MCLK = SMCLK = CALxxx_1MHZ = 1MHz
//
//                MSP430G2xx3
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |     P1.2/UCA0TXD|------------>
//            |                 | 9600 - 8N1
//            |     P1.1/UCA0RXD|<------------
//
//   D. Dang
//   Texas Instruments Inc.
//   February 2011
//   Built with CCS Version 4.2.0 and IAR Embedded Workbench Version: 5.10
//*****************************************************************************
#include  "main.h"

UARTline_t UARTlines[MAX_UARTlines];
unsigned char UARTline[UARTline_max];
unsigned int UARTline_index=0;
unsigned int UART_write_index=0;
unsigned int UART_read_index=0;

//
void vUART_init()
{
	  UCA0CTL0 = 0;	// no Parity, LSB first, 8-Bit data, 1 Stopbit
	  UCA0CTL1 |= UCSSEL_2 + UCRXEIE + UCSWRST;	// UART Software Reset, SMCLK
	  P1SEL = BIT1 + BIT2;						// use Pins for UART-Function
	  P1SEL2 = BIT1 + BIT2;						//  P1.1 = RXD, P1.2=TXD
	  UCA0CTL1 |= UCSSEL_2;						// SMCLK, auch fehlerhafte Zeichen lsen Interrupt aus

	  // Baudrate-Generator siehe MSP430x2xxx Family Guide 15.3.13  Table 15-4 Typical Baud Rates and Errors
#if (F_MHZ == 1)
	  UCA0BR0 = 8;								// 115200 Bd @1MHz Clock
	  UCA0BR1 = 0;
	  UCA0MCTL = UCBRS2 + UCBRS1;				// Modulation UCBRSx = 6
#endif

#if (F_MHZ == 8)
	  UCA0BR0 = 69;								// 115200 Bd @8MHz Clock
	  UCA0BR1 = 0;
	  UCA0MCTL = UCBRS1;						// Modulation UCBRSx = 4
#endif

#if (F_MHZ == 12)
	  UCA0BR0 = 104;							// 115200 Bd @12MHz Clock
	  UCA0BR1 = 0;
	  UCA0MCTL = UCBRS1;						// Modulation UCBRSx = 1
#endif
#if 1
#if (F_MHZ == 16)
	  UCA0BR1 = 0x06;							// 1666 =>9600 Bd @16MHz Clock
	  UCA0BR0 = 0x82;
	  UCA0MCTL = UCBRS2 + UCBRS1;				// Modulation UCBRSx = 6
#endif
#else
#if (F_MHZ == 16)
	  UCA0BR0 = 138;							// 138 =>115200 Bd @16MHz Clock
	  UCA0BR1 = 0;
	  UCA0MCTL = UCBRS2 + UCBRS1 + UCBRS0;		// Modulation UCBRSx = 7
#endif
#endif

	  UCA0CTL1 &= ~UCSWRST;						// **Initialize USCI state machine**
	  IE2 |= UCA0RXIE;							// enable RX-Interrupt
//	  __bis_SR_register(GIE);					// interrupts globally enabled -> macht das Hauptprogramm
}

void vUART_sendChar(char cc) {
	while(!(UCA0TXIFG & IFG2)) ;		// USCI_A0 TX buffer ready?
	UCA0TXBUF = cc;
}

void vUART_sendChars(unsigned int count, unsigned char* chars) {
	unsigned int index;
	for(index = 0; index < count; ++index) {
		vUART_sendChar(chars[index]);
	}
}


//UART-Rx Interrupt-Service-Routine fr GPS-RX
// neues Zeichen in den aktuellen Zeilenbuffer aufnehmen
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void) {
	// Zeichen im Buffer sichern und testen, ob mit LF das Zeilenende erreicht ist
	if((UARTlines[UARTline_index].zeile[UART_write_index] = UCA0RXBUF) == '\n') {
		UARTlines[UARTline_index].valid = Ja;	// die Zeile ist vollstndig
		if(++UARTline_index >= MAX_UARTlines) {	// nchste Bufferzeile aktivieren
			UARTline_index = 0;
		}
		UART_write_index = 0;
		UARTlines[UARTline_index].valid = Nein;	// die neue Zeile ist noch unvollstndig
		return;
	}
	if(++UART_write_index >= UARTline_max)	// Bufferende erreicht?
		UART_write_index = 0;		// tja schade...
}

// Ende USCIA0_UART.c
