#include <msp430.h> 

/*
 * main.c
 */

#include "main.h"


SystemStatus_t SystemStatus;
unsigned int SD_card_write();

void main(void) {
	vCPU_init();
	vI2C_init();
	SystemStatus.devices.LCDok		= jnLCD_init();
	SystemStatus.devices.BMP180ok	= jnBMP180_init();
	vUART_init();				// 9600 Baud fr ultimate GPS, RX-Interrupts enabled
	_bis_SR_register( GIE);		//enable interrupts globally

	while(!GPS_init())			// adafruit ultimate GPS breakout v3
		SystemStatus.devices.GPS_RXok = Nein;
	SystemStatus.devices.GPS_RXok = Ja;

#ifdef LCD
	vLCD_puts("GPSRx 09.02.2015");
	_delay_ms(1000);
#endif

#if 0									// testen der _delay-Routinen
	unsigned int count;
	for(count=0;count <250; ++count) {	// per _delay_us 10 Sekunden warten
		_delay_us(40000);
	}
	_delay_ms(10000);					// per _delay_ms 10 Sekunden warten
#endif

	while(42) {
		unsigned int error_code;

		SystemStatus.messwerte.bmp180.temp_x_10 = iBMP180_get_temperature(&error_code);
		if(!error_code)
			SystemStatus.messwerte.bmp180.luftdruck = liBMP180_get_pressure(&error_code);
#ifdef LCD
		vLCD_gotoxy(10,2);				// BMP180-Luftdruck anzeigen
		vLCD_putli(SystemStatus.messwerte.bmp180.luftdruck, 6, 2);
#endif
		GPS_check();
		vLCD_gotoxy(7,1);
		vLCD_putnC(SystemStatus.messwerte.GPS_RX.date, 2);
		vLCD_putc('.');
		vLCD_putnC(SystemStatus.messwerte.GPS_RX.date+2, 2);
		vLCD_puts(".20");
		vLCD_putnC(SystemStatus.messwerte.GPS_RX.date+4, 2);
		vLCD_gotoxy(1,2);
		vLCD_putnC(SystemStatus.messwerte.GPS_RX.UTC, 2);
		vLCD_putc(':');
		vLCD_putnC(SystemStatus.messwerte.GPS_RX.UTC+2, 2);
		vLCD_putc(':');
		vLCD_putnC(SystemStatus.messwerte.GPS_RX.UTC+4, 2);
	}
}


void vCPU_init(void) {
	WDTCTL = WDTPW + WDTHOLD;	//stoppe Watchdog Timer
								//initialize Basic Clock Module
#if (F_MHZ == 8)
	// fr MCLK wird der interne Oszillator verwendet	=> Set DCO to 8 MHz
	DCOCTL = CALDCO_8MHZ;
	BCSCTL1 = CALBC1_8MHZ;
	BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;
#endif
#if (F_MHZ == 12)
	// fr MCLK wird der interne Oszillator verwendet	=> Set DCO to 4 MHz
//	DCOCTL = 0;								// Select lowest DCOx and MODx settings
	BCSCTL1 = CALBC1_12MHZ;					// Set range
	DCOCTL= CALDCO_12MHZ;					// Set DCO step + modulation
	BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;
#endif
#if (F_MHZ == 16)
	// fr MCLK wird der interne Oszillator verwendet	=> Set DCO to 16 MHz
	DCOCTL = CALDCO_16MHZ;
	BCSCTL1 = CALBC1_16MHZ;
	BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;
#endif
}

// msec Millisekunden warten
void _delay_ms(unsigned int msec) {
  volatile unsigned int count;
	while(msec--) {
#if (F_MHZ == 8)
		count = 624;	// passt bei 8MHz aus dem internen Oszillator DCO
#endif
#if (F_MHZ == 12)
		count = 1200;	// passt bei 12MHz aus dem internen Oszillator DCO
#endif
#if (F_MHZ == 16)
		count = 1600;	// passt bei 16MHz aus dem internen Oszillator DCO
#endif
		while(count) {
			--count;
		}
	}
}
#if (F_MHZ == 8)
// usec Mikrosekunden warten (ungefhr...)
void _delay_us(volatile unsigned int usec) {
	while(usec--) {
	}
}
#endif
#if (F_MHZ == 12)
// usec Mikrosekunden warten (ungefhr...)
void _delay_us(volatile unsigned int usec) {
	while(usec--) {
		_NOP();
		_NOP();
		_NOP();
	}
}
#endif
#if (F_MHZ == 16)
// usec Mikrosekunden warten (ungefhr...)
void _delay_us(volatile unsigned int usec) {
	while(usec--) {
		_NOP();
		_NOP();
		_NOP();
		_NOP();
		_NOP();
		_NOP();
	}
}
#endif

// Ende main.c
